@@ -181,11 +181,9 @@ struct _YelpWindowPrivate {
     gint height;
 
     gboolean configured;
-
-    gboolean use_header;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (YelpWindow, yelp_window, GTK_TYPE_APPLICATION_WINDOW)
+G_DEFINE_TYPE_WITH_PRIVATE (YelpWindow, yelp_window, HDY_TYPE_APPLICATION_WINDOW)
 
 static void
 yelp_window_init (YelpWindow *window)
@@ -284,6 +282,27 @@ yelp_window_set_property (GObject     *object,
     }
 }
 
+static gboolean
+window_button_press (YelpWindow *window, GdkEventButton *event, gpointer userdata)
+{
+    switch (event->button) {
+        case 8:
+            g_action_activate (g_action_map_lookup_action (G_ACTION_MAP (window),
+                                                           "yelp-view-go-back"),
+                               NULL);
+            return TRUE;
+
+        case 9:
+            g_action_activate (g_action_map_lookup_action (G_ACTION_MAP (window),
+                                                           "yelp-view-go-forward"),
+                               NULL);
+            return TRUE;
+
+        default:
+            return FALSE;
+    }
+}
+
 static void
 window_construct (YelpWindow *window)
 {
@@ -293,7 +312,6 @@ window_construct (YelpWindow *window)
     GtkSizeGroup *size_group;
     GMenu *menu, *section;
     YelpWindowPrivate *priv = yelp_window_get_instance_private (window);
-    GtkStyleContext *headerbar_context;
 
     const GActionEntry entries[] = {
         { "yelp-window-new",    action_new_window,   NULL, NULL, NULL },
@@ -306,35 +324,23 @@ window_construct (YelpWindow *window)
 
     gtk_window_set_icon_name (GTK_WINDOW (window), "org.gnome.Yelp");
 
-    g_object_get (gtk_settings_get_default (),
-                  "gtk-dialogs-use-header", &priv->use_header,
-                  NULL);
-
     priv->view = (YelpView *) yelp_view_new ();
 
     g_action_map_add_action_entries (G_ACTION_MAP (window),
                                      entries, G_N_ELEMENTS (entries), window);
     yelp_view_register_actions (priv->view, G_ACTION_MAP (window));
 
-    priv->vbox_full = gtk_box_new (GTK_ORIENTATION_VERTICAL, 3);
+    priv->vbox_full = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_add (GTK_CONTAINER (window), priv->vbox_full);
 
-    priv->header = gtk_header_bar_new ();
-    if (priv->use_header) {
-        gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->header), TRUE);
-        gtk_window_set_titlebar (GTK_WINDOW (window), priv->header);
-    } else {
-        headerbar_context = gtk_widget_get_style_context (GTK_WIDGET (priv->header));
-        gtk_container_add (GTK_CONTAINER (priv->vbox_full), GTK_WIDGET (priv->header));
-        gtk_style_context_remove_class (headerbar_context, "header-bar");
-        gtk_style_context_add_class (headerbar_context, GTK_STYLE_CLASS_TOOLBAR);
-        gtk_style_context_add_class (headerbar_context, GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
-    }
+    priv->header = hdy_header_bar_new ();
+    hdy_header_bar_set_show_close_button (HDY_HEADER_BAR (priv->header), TRUE);
+    gtk_container_add (GTK_CONTAINER (priv->vbox_full), priv->header);
 
     /** Back/Forward **/
     box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_style_context_add_class (gtk_widget_get_style_context (box), "linked");
-    gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->header), box);
+    hdy_header_bar_pack_start (HDY_HEADER_BAR (priv->header), box);
 
     button = gtk_button_new_from_icon_name ("go-previous-symbolic", GTK_ICON_SIZE_MENU);
     gtk_widget_set_tooltip_text (button, _("Back"));
@@ -356,7 +362,7 @@ window_construct (YelpWindow *window)
     gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
     gtk_style_context_add_class (gtk_widget_get_style_context (button), "image-button");
     gtk_widget_set_tooltip_text (button, _("Menu"));
-    gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->header), button);
+    hdy_header_bar_pack_end (HDY_HEADER_BAR (priv->header), button);
 
     menu = g_menu_new ();
     section = g_menu_new ();
@@ -407,7 +413,7 @@ window_construct (YelpWindow *window)
                             G_BINDING_BIDIRECTIONAL);
     g_signal_connect (priv->search_bar, "notify::search-mode-enabled",
                       G_CALLBACK (window_search_mode), window);
-    gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->header), button);
+    hdy_header_bar_pack_end (HDY_HEADER_BAR (priv->header), button);
 
     g_signal_connect (window, "key-press-event", G_CALLBACK (window_key_press), NULL);
 
@@ -419,7 +425,7 @@ window_construct (YelpWindow *window)
                           gtk_image_new_from_icon_name ("user-bookmarks-symbolic",
                                                         GTK_ICON_SIZE_MENU));
     gtk_widget_set_tooltip_text (button, _("Bookmarks"));
-    gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->header), button);
+    hdy_header_bar_pack_end (HDY_HEADER_BAR (priv->header), button);
 
     priv->bookmark_menu = gtk_popover_new (button);
     g_object_set (priv->bookmark_menu, "border-width", 12, NULL);
@@ -543,6 +549,8 @@ window_construct (YelpWindow *window)
     gtk_drag_dest_add_uri_targets (GTK_WIDGET (window));
     g_signal_connect (window, "drag-data-received",
                       G_CALLBACK (window_drag_received), NULL);
+
+    g_signal_connect (window, "button-press-event", G_CALLBACK (window_button_press), NULL);
 }
 
 /******************************************************************************/
@@ -670,7 +678,7 @@ action_ctrll (GSimpleAction *action,
 
     gtk_entry_set_text (GTK_ENTRY (priv->ctrll_entry), "");
 
-    gtk_header_bar_set_custom_title (GTK_HEADER_BAR (priv->header), priv->ctrll_entry);
+    hdy_header_bar_set_custom_title (HDY_HEADER_BAR (priv->header), priv->ctrll_entry);
     gtk_widget_show (priv->ctrll_entry);
     gtk_widget_grab_focus (priv->ctrll_entry);
 
@@ -775,7 +783,7 @@ window_key_press (YelpWindow  *window,
     if (gtk_revealer_get_reveal_child (GTK_REVEALER (priv->find_bar)))
         return FALSE;
 
-    if (gtk_header_bar_get_custom_title (GTK_HEADER_BAR (priv->header)))
+    if (hdy_header_bar_get_custom_title (HDY_HEADER_BAR (priv->header)))
         return FALSE;
 
     return gtk_search_bar_handle_event (GTK_SEARCH_BAR (priv->search_bar),
@@ -1178,28 +1186,15 @@ view_root_title (YelpView    *view,
     gchar *root_title, *page_title;
     g_object_get (view, "root-title", &root_title, "page-title", &page_title, NULL);
 
-    if (!priv->use_header) {
-        if (root_title)
-            gtk_window_set_title (GTK_WINDOW (window), root_title);
-        else
-            gtk_window_set_title (GTK_WINDOW (window), _("Help"));
-
-        goto out;
-    }
-
     if (page_title)
-        gtk_header_bar_set_title (GTK_HEADER_BAR (priv->header), page_title);
+        hdy_header_bar_set_title (HDY_HEADER_BAR (priv->header), page_title);
     else
-        gtk_header_bar_set_title (GTK_HEADER_BAR (priv->header), _("Help"));
+        hdy_header_bar_set_title (HDY_HEADER_BAR (priv->header), _("Help"));
 
     if (root_title && (page_title == NULL || strcmp (root_title, page_title)))
-        gtk_header_bar_set_subtitle (GTK_HEADER_BAR (priv->header), root_title);
+        hdy_header_bar_set_subtitle (HDY_HEADER_BAR (priv->header), root_title);
     else
-        gtk_header_bar_set_subtitle (GTK_HEADER_BAR (priv->header), NULL);
-
-out:
-    g_free (root_title);
-    g_free (page_title);
+        hdy_header_bar_set_subtitle (HDY_HEADER_BAR (priv->header), NULL);
 }
 
 static void
@@ -1212,7 +1207,7 @@ ctrll_entry_activate (GtkEntry    *entry,
     yelp_window_load_uri (window, uri);
     g_object_unref (uri);
 
-    gtk_header_bar_set_custom_title (GTK_HEADER_BAR (priv->header), NULL);
+    hdy_header_bar_set_custom_title (HDY_HEADER_BAR (priv->header), NULL);
 }
 
 static gboolean
@@ -1223,7 +1218,7 @@ ctrll_entry_key_press (GtkWidget    *widget,
     YelpWindowPrivate *priv = yelp_window_get_instance_private (window);
 
     if (event->keyval == GDK_KEY_Escape) {
-        gtk_header_bar_set_custom_title (GTK_HEADER_BAR (priv->header), NULL);
+        hdy_header_bar_set_custom_title (HDY_HEADER_BAR (priv->header), NULL);
         return TRUE;
     }
     return FALSE;
